searchNodes=[{"ref":"Incident.Aggregate.html","title":"Incident.Aggregate","type":"behaviour","doc":"An aggregate is the unit of business logic in the domain. The business logic is used to handle commands and also to apply events to change the aggregate state. This behaviour defines the callbacks for command execution and event application."},{"ref":"Incident.Aggregate.html#c:apply/2","title":"Incident.Aggregate.apply/2","type":"callback","doc":"Receives a persisted event and the aggregate state, performing an aggregate state update."},{"ref":"Incident.Aggregate.html#c:execute/1","title":"Incident.Aggregate.execute/1","type":"callback","doc":"Receives and executes the command, performing a specific business logic for it, usually considering the aggregate state as part of the logic. In case of a successful command and logic, a new event is composed and returned along with the aggregate state. In case of an unsuccessful command and logic, an error is returned containing a business logic reason."},{"ref":"Incident.AggregateState.html","title":"Incident.AggregateState","type":"module","doc":"Defines an Aggregate State macro that receives the aggregate module and an initial state."},{"ref":"Incident.Command.html","title":"Incident.Command","type":"behaviour","doc":"Defines the API for a Command."},{"ref":"Incident.Command.html#c:valid?/1","title":"Incident.Command.valid?/1","type":"callback","doc":"Returns if a command is valid or not."},{"ref":"Incident.CommandHandler.html","title":"Incident.CommandHandler","type":"module","doc":"Defines a Command Handler that receives a command to: ensure that the command data is valid based on the command validations; executes the command using the aggregate;"},{"ref":"Incident.Event.PersistedEvent.html","title":"Incident.Event.PersistedEvent","type":"module","doc":"Defines the common data structure for any event that is persisted in the Event Store. All fields are required."},{"ref":"Incident.Event.PersistedEvent.html#t:t/0","title":"Incident.Event.PersistedEvent.t/0","type":"type","doc":""},{"ref":"Incident.EventHandler.html","title":"Incident.EventHandler","type":"behaviour","doc":"Defines the API for an Event Handler."},{"ref":"Incident.EventHandler.html#c:listen/2","title":"Incident.EventHandler.listen/2","type":"callback","doc":"Listens to a persisted event and an aggregate state. You can use the aggregate to apply the persisted event and its state to get a new state, and to project new data into the projection store."},{"ref":"Incident.EventStore.html","title":"Incident.EventStore","type":"module","doc":"Defines the API to interact with the Event Store. The data source is based on the configured Event Store Adapter."},{"ref":"Incident.EventStore.Adapter.html","title":"Incident.EventStore.Adapter","type":"behaviour","doc":"Defines the API for an Event Store adapter."},{"ref":"Incident.EventStore.Adapter.html#c:append/1","title":"Incident.EventStore.Adapter.append/1","type":"callback","doc":"Appends an event to the Event Store."},{"ref":"Incident.EventStore.Adapter.html#c:get/1","title":"Incident.EventStore.Adapter.get/1","type":"callback","doc":"Receives an aggregate id and returns a list containing all persisted events from the Event Store."},{"ref":"Incident.EventStore.InMemoryAdapter.html","title":"Incident.EventStore.InMemoryAdapter","type":"module","doc":"Implements an in-memory Event Store using Agents."},{"ref":"Incident.EventStore.InMemoryAdapter.html#child_spec/1","title":"Incident.EventStore.InMemoryAdapter.child_spec/1","type":"function","doc":"Returns a specification to start this module under a supervisor. See Supervisor."},{"ref":"Incident.EventStore.InMemoryAdapter.html#start_link/1","title":"Incident.EventStore.InMemoryAdapter.start_link/1","type":"function","doc":""},{"ref":"Incident.ProjectionStore.html","title":"Incident.ProjectionStore","type":"module","doc":"Defines the API to interact with the Projection Store. The data source is based on the configured Projection Store Adapter."},{"ref":"Incident.ProjectionStore.Adapter.html","title":"Incident.ProjectionStore.Adapter","type":"behaviour","doc":"Defines the API for a Projection Store adapter."},{"ref":"Incident.ProjectionStore.Adapter.html#c:all/1","title":"Incident.ProjectionStore.Adapter.all/1","type":"callback","doc":"Returns all projections from a specific projection type from the Projection Store."},{"ref":"Incident.ProjectionStore.Adapter.html#c:project/2","title":"Incident.ProjectionStore.Adapter.project/2","type":"callback","doc":"Insert or updates a projection in the Projection Store. Receives the projection type and the data for the projection."},{"ref":"Incident.ProjectionStore.InMemoryAdapter.html","title":"Incident.ProjectionStore.InMemoryAdapter","type":"module","doc":"Implements an in-memory Projection Store using Agents."},{"ref":"Incident.ProjectionStore.InMemoryAdapter.html#child_spec/1","title":"Incident.ProjectionStore.InMemoryAdapter.child_spec/1","type":"function","doc":"Returns a specification to start this module under a supervisor. See Supervisor."},{"ref":"Incident.ProjectionStore.InMemoryAdapter.html#start_link/1","title":"Incident.ProjectionStore.InMemoryAdapter.start_link/1","type":"function","doc":""},{"ref":"readme.html","title":"Incident","type":"extras","doc":"Incident Event Sourcing and CQRS in Elixir abstractions."},{"ref":"readme.html#goals","title":"Incident - Goals","type":"extras","doc":"incentivize the usage of Event Sourcing and CQRS as good choice for domains that can leverage the main benefits of this design pattern; serve as guidance when using Event Sourcing in your system; leverage functions and reducers for executing commands and applying events; allow customization for fine-grained needs without compromising the principles;"},{"ref":"readme.html#event-sourcing-and-cqrs","title":"Incident - Event Sourcing and CQRS","type":"extras","doc":"In a nutshell, Event Sourcing ensures that all changes to application state are stored as a sequence of events. But if you are new to Event Sourcing and CQRS I highly recommend watch Greg Young&#39;s presentation at Code on the Beach 2014 before moving forward."},{"ref":"readme.html#main-components","title":"Incident - Main Components","type":"extras","doc":"Command is a data structure with basic validation; Command Handler is the entry point of a command in the write side, it performs basic command validation and executes the command through the aggregate; Aggregate only defines its business logic to execute a command and apply an event, but not its state; Aggregate State is defined by playing all the events through Aggregate business logic; Event is a data structure; Event Handler receives a persisted event and performs the actions in the read side; Event Store and Projection Store are swappable persistence layers to allow different technologies over time; Projection can be rebuilt based on the persisted events and on the same Aggregate business logic;"},{"ref":"readme.html#roadmap","title":"Incident - Roadmap","type":"extras","doc":"Next Steps [ ] add Postgres as an option for event and projection storage via a built-in Ecto Adapter; Done [x] create a Proof of Concept that can exercise all library components, implementation and confirm goals; [x] extract library components from the POC, remove implementation and document the implementation in the README; [x] add example application within the library as a reference for newcomers; [x] publish version 0.1.0 to Hex and make repository public; [x] define behaviour for event handlers; [x] define a macro for command handlers; [x] define behaviour for commands; [x] validate commands in the command handlers using command implementation; [x] use Ecto.Schema for command data structure; [x] use Ecto.Schema event data structure; [x] move event persistency from aggregates to command handlers;"},{"ref":"readme.html#installation","title":"Incident - Installation","type":"extras","doc":"If available in Hex, the package can be installed by adding incident to your list of dependencies in mix.exs: def deps do [ {:incident, &quot;~&gt; 0.2.0&quot;} ] end"},{"ref":"readme.html#configuration","title":"Incident - Configuration","type":"extras","doc":"Event Store and Projection Store Setup Configure incident Event Store and Projection Store adapters and if desired, some options. The options will be passed in during the adapter initialization. config :incident, :event_store, adapter: Incident.EventStore.InMemoryAdapter, options: [ initial_state: [] ] config :incident, :projection_store, adapter: Incident.ProjectionStore.InMemoryAdapter, options: [ initial_state: %{} ]"},{"ref":"readme.html#getting-started","title":"Incident - Getting Started","type":"extras","doc":"There is an example application that implements a Bank application for reference, including all the details and usage in IEx as well."},{"ref":"readme.html#documentation","title":"Incident - Documentation","type":"extras","doc":"Documentation can be generated with ExDoc and published on HexDocs. Once published, the docs can be found at https://hexdocs.pm/incident."}]